@@ -4,7 +4,7 @@
 # SOAP::Lite is free software; you can redistribute it
 # and/or modify it under the same terms as Perl itself.
 #
-# $Id: SOAP::Transport::HTTP.pm,v 0.40 2000/10/15 18:20:55 $
+# $Id: SOAP::Transport::HTTP.pm,v 0.43 2000/11/28 01:47:02 $ 
 #
 # ======================================================================
 
@@ -12,45 +12,58 @@ package SOAP::Transport::HTTP;
 
 use strict;
 use vars qw($VERSION);
-$VERSION = '0.40';
+$VERSION = '0.43';
 
 # ======================================================================
 
 package SOAP::Transport::HTTP::Client;
 
 use vars qw(@ISA);
-@ISA = qw(LWP::UserAgent);
+@ISA = qw(SOAP::Client LWP::UserAgent);
 
-sub new { eval "use LWP::UserAgent; use URI"; die if $@;
+use SOAP::Lite;
+
+my(%redirect, %mpost);
+
+# hack for HTTP conection that returns Keep-Alive 
+# miscommunication (?) between LWP::Protocol and LWP::Protocol::http
+# die after timeout
+sub patch { 
+  local $^W; 
+  eval "package LWP::UserAgent; sub redirect_ok {1}";
+  { package LWP::Protocol; 
+    my $collect = \&collect; # store original  
+    *collect = sub {          
+      if (defined $_[2]->header('Connection') && $_[2]->header('Connection') eq 'Keep-Alive') {
+        my $data = $_[3]->(); 
+        my $next = length($$data) == $_[2]->header('Content-Length') ? sub { \'' } : $_[3];
+        my $done = 0; $_[3] = sub { $done++ ? &$next : $data };
+      }
+      goto &$collect;
+    };
+  }
+  *patch = sub {};
+};
+
+sub DESTROY { SOAP::Trace::objects('()') }
+
+sub new { require LWP::UserAgent; patch;
   my $self = shift;
   my $class = ref($self) || $self;
 
   unless (ref $self) {
-    $self = bless $class->SUPER::new() => $class;
-    $self->agent(join '/', 'SOAP::Transport', 'Perl', SOAP::Transport::HTTP->VERSION);
-    $self->on_debug(sub {});
-  }
-
-  if (@_) {
-    my %parameters = @_;
-    foreach (grep {defined $parameters{$_}} keys %parameters) {
-      $self->$_($parameters{$_}) if $self->can($_);
+    my(@params, @methods);
+    while (@_) { $class->can($_[0]) ? push(@methods, shift() => shift) : push(@params, shift) }
+    $self = $class->SUPER::new(@params);
+    $self->agent(join '/', 'SOAP::Lite', 'Perl', SOAP::Transport::HTTP->VERSION);
+    while (@methods) { my($method, $params) = splice(@methods,0,2);
+      $self->$method(ref $params eq 'ARRAY' ? @$params : $params) 
     }
+    SOAP::Trace::objects('()');
   }
   return $self;
 }
 
-sub BEGIN {
-  no strict 'refs';
-  for my $method (qw(endpoint code message is_success status on_debug parameters)) {
-    my $field = '_' . $method;
-    *$method = sub {
-      my $self = shift->new;
-      @_ ? ($self->{$field} = shift, return $self) : return $self->{$field};
-    }
-  }
-}
-
 sub send_receive {
   my($self, %parameters) = @_;
   my($envelope, $endpoint, $action) = 
@@ -58,24 +71,50 @@ sub send_receive {
 
   $endpoint ||= $self->endpoint;
 
-  my $req = HTTP::Request->new(POST => $endpoint, HTTP::Headers->new, $envelope);
+  my $method = 'POST';
+  my $resp;
+  while (1) { 
 
-  $req->header('SOAPAction' => $action);
-  $req->header(Host => URI->new($self->endpoint)->host_port);
-  $req->content_type('text/xml');
-  $req->content_length(length($envelope));
+    # check cache for redirect
+    $endpoint = $redirect{$endpoint} if exists $redirect{$endpoint};
+    # check cache for M-POST
+    $method = 'M-POST' if exists $mpost{$endpoint};
 
-  $self->on_debug->($req->as_string);
+    my $req = HTTP::Request->new($method => $endpoint, HTTP::Headers->new, $envelope);
+    $req->proxy_authorization_basic($ENV{'HTTP_proxy_user'}, $ENV{'HTTP_proxy_pass'})
+      if ($ENV{'HTTP_proxy_user'} && $ENV{'HTTP_proxy_pass'});
+
+    if ($method eq 'M-POST') {
+      my $prefix = sprintf '%04d', int(rand(1000));
+      $req->header(Man => qq!"$SOAP::Constants::NS_ENV"; ns=$prefix!);
+      $req->header("$prefix-SOAPAction" => $action);  
+    } else {
+      $req->header(SOAPAction => $action);
+    }
+    $req->content_type('text/xml');
+    $req->content_length(length($envelope));
+
+    SOAP::Trace::transport($req);
+    SOAP::Trace::debug($req->as_string);
     
-  my $resp = $self->SUPER::request($req);
+    $self->SUPER::env_proxy if $ENV{'HTTP_proxy'};
 
-  $self->on_debug->($resp->as_string);
+    $resp = $self->SUPER::request($req);
 
-# 200 OK
-# 204 OK, one-way method
-# 3?? Redirects, cache results
-# 405/510 POST not supported, try M-POST
-#+ 500 Server/Client side error, content can be present
+    SOAP::Trace::transport($resp);
+    SOAP::Trace::debug($resp->as_string);
+
+    # 100 OK, continue to read?
+    if (($resp->code == 510 || $resp->code == 501 || $resp->code == 405) && 
+        $method ne 'M-POST') { 
+      $mpost{$endpoint} = 1;
+    } else {
+      last;
+    }
+  }
+
+  $redirect{$endpoint} = $resp->request->url
+    if $resp->previous && $resp->previous->is_redirect;
 
   $self->code($resp->code);
   $self->message($resp->message);
@@ -89,28 +128,25 @@ sub send_receive {
 
 package SOAP::Transport::HTTP::Server;
 
-use SOAP::Lite;
 use vars qw(@ISA);
 @ISA = qw(SOAP::Server);
 
-sub new { 
+use SOAP::Lite;
+
+sub DESTROY { SOAP::Trace::objects('()') }
+
+sub new { require LWP::UserAgent;
   my $self = shift;
   my $class = ref($self) || $self;
 
   unless (ref $self) {
-    $self = bless $class->SUPER::new() => $class;
+    $self = $class->SUPER::new(@_);
     $self->on_action(sub {
-      (my $action = shift) =~ s/^("?)(.+)\1$/$2/;
+      (my $action = shift) =~ s/^("?)(.*)\1$/$2/;
       die "SOAPAction shall match 'uri#method' if present\n" 
-        if $action && $action ne join '#', @_;
+        if $action && $action ne join('#', @_) && $action ne join('/', @_);
     });
-  }
-
-  if (@_) {
-    my %parameters = @_;
-    foreach (grep {defined $parameters{$_}} keys %parameters) {
-      $self->$_($parameters{$_}) if $self->can($_);
-    }
+    SOAP::Trace::objects('()');
   }
   return $self;
 }
@@ -129,32 +165,44 @@ sub BEGIN {
 sub handle {
   my $self = shift->new;
 
-  return $self->response(HTTP::Response->new(400)) # BAD_REQUEST
-    unless $self->request->method eq 'POST';
+  $self->myuri($self->request->uri->canonical);
+
+  if ($self->request->method eq 'POST') {
+    $self->action($self->request->header('SOAPAction'));
+  } elsif ($self->request->method eq 'M-POST') {
+    return $self->response(HTTP::Response->new(510, # NOT EXTENDED
+           "Expected Mandatory header with $SOAP::Constants::NS_ENV as unique URI")) 
+      if $self->request->header('Man') !~ /^"$SOAP::Constants::NS_ENV";\s*ns\s*=\s*(\d+)/;
+    $self->action($self->request->header("$1-SOAPAction"));
+  } else {
+    return $self->response(HTTP::Response->new(405)) # METHOD NOT ALLOWED
+  }
 
   return $self->make_fault($SOAP::Constants::FAULT_CLIENT, 'Bad Request' => 'Content-Type must be text/xml')
     unless $self->request->content_type eq 'text/xml';
 
-  $self->action($self->request->header('SOAPAction'));
-
   my $response = $self->SUPER::handle($self->request->content) or return;
 
-  $self->response(HTTP::Response->new( 
-     $SOAP::Constants::HTTP_ON_SUCCESS_CODE => undef, 
-     HTTP::Headers->new('Content-Type' => 'text/xml', 'Content-Length' => length $response), 
-     $response,
-  ));
+  $self->make_response($SOAP::Constants::HTTP_ON_SUCCESS_CODE, $response);
 }
 
 sub make_fault {
   my $self = shift;
-  my $response = $self->SUPER::make_fault(@_);
-  $self->response(HTTP::Response->new(
-     $SOAP::Constants::HTTP_ON_FAULT_CODE => undef,
-     HTTP::Headers->new('Content-Type' => 'text/xml', 'Content-Length' => length $response),
+  $self->make_response($SOAP::Constants::HTTP_ON_FAULT_CODE => $self->SUPER::make_fault(@_));
+  return;
+}
+
+sub make_response {
+  my $self = shift;
+  my($code, $response) = @_;
+  $self->response(HTTP::Response->new( 
+     $code => undef, 
+     HTTP::Headers->new(
+       'SOAPServer' => join('/', 'SOAP::Lite', 'Perl', SOAP::Transport::HTTP->VERSION),
+       'Content-Type' => 'text/xml', 
+       'Content-Length' => length $response), 
      $response,
   ));
-  return;
 }
 
 # ======================================================================
@@ -164,11 +212,15 @@ package SOAP::Transport::HTTP::CGI;
 use vars qw(@ISA);
 @ISA = qw(SOAP::Transport::HTTP::Server);
 
+sub DESTROY { SOAP::Trace::objects('()') }
+
 sub new { 
   my $self = shift;
   my $class = ref($self) || $self;
+
   unless (ref $self) {
-    $self = bless $class->SUPER::new() => $class;
+    $self = $class->SUPER::new(@_);
+    SOAP::Trace::objects('()');
   }
   return $self;
 }
@@ -179,10 +231,7 @@ sub handle {
   my $content; read(STDIN,$content,$ENV{'CONTENT_LENGTH'});  
   $self->request(HTTP::Request->new( 
     $ENV{'REQUEST_METHOD'} => $ENV{'SCRIPT_NAME'},
-    HTTP::Headers->new(
-      map { (my $http = uc $_) =~ s/-/_/g; $_ => $ENV{"HTTP_$http"} || $ENV{$http};
-          } qw(Content-Type SOAPAction),
-    ),
+    HTTP::Headers->new(map {(/^HTTP_(.+)/i ? $1 : $_) => $ENV{$_}} keys %ENV),
     $content,
   ));
   $self->SUPER::handle;
@@ -198,21 +247,26 @@ sub handle {
 
 package SOAP::Transport::HTTP::Daemon;
 
+use Carp;
 use vars qw($AUTOLOAD @ISA);
 @ISA = qw(SOAP::Transport::HTTP::Server);
 
-sub new { eval "use HTTP::Daemon"; die if $@;
+sub DESTROY { SOAP::Trace::objects('()') }
+
+sub new { require HTTP::Daemon; 
   my $self = shift;
   my $class = ref($self) || $self;
+
   unless (ref $self) {
-    $self = bless $class->SUPER::new() => $class;
-    $self->{_daemon} = HTTP::Daemon->new(@_);
+    $self = $class->SUPER::new();
+    $self->{_daemon} = HTTP::Daemon->new(@_) or croak "Can't create daemon: $!";
+    SOAP::Trace::objects('()');
   }
   return $self;
 }
 
 sub AUTOLOAD {
-  my($method) = $AUTOLOAD =~ m/([^:]+)$/;
+  my $method = substr($AUTOLOAD, rindex($AUTOLOAD, '::') + 2);
   return if $method eq 'DESTROY';
 
   no strict 'refs';
@@ -223,13 +277,13 @@ sub AUTOLOAD {
 sub handle {
   my $self = shift->new;
   while (my $c = $self->accept) {
-      while (my $r = $c->get_request) {
-          $self->request($r);
-          $self->SUPER::handle;
-          $c->send_response($self->response)
-      }
-      $c->close;
-      undef $c;
+    while (my $r = $c->get_request) {
+      $self->request($r);
+      $self->SUPER::handle;
+      $c->send_response($self->response)
+    }
+    $c->close;
+    undef $c;
   }
 }
 
@@ -240,25 +294,26 @@ package SOAP::Transport::HTTP::Apache;
 use vars qw(@ISA);
 @ISA = qw(SOAP::Transport::HTTP::Server);
 
-sub new { 
+sub DESTROY { SOAP::Trace::objects('()') }
+
+sub new { require Apache; require Apache::Constants;
   my $self = shift;
   my $class = ref($self) || $self;
+
   unless (ref $self) {
-    $self = bless $class->SUPER::new() => $class;
+    $self = $class->SUPER::new(@_);
+    SOAP::Trace::objects('()');
   }
   return $self;
 }
 
-sub handler { eval "use Apache; use Apache::Constants qw(OK)"; die if $@;
+sub handler { 
   my $self = shift->new; 
   my $r = shift || Apache->request; 
 
   $self->request(HTTP::Request->new( 
     $r->method => $r->uri,
-    HTTP::Headers->new(
-      'Content-Type' => $r->header_in('Content-type'),
-      'SOAPAction' => $r->header_in('SOAPAction'),
-    ),
+    HTTP::Headers->new($r->headers_in),
     do { my $buf; $r->read($buf, $r->header_in('Content-length')); $buf; } 
   ));
   $self->SUPER::handle;
@@ -270,7 +325,7 @@ sub handler { eval "use Apache; use Apache::Constants qw(OK)"; die if $@;
   $r->send_http_header;
   $r->print($self->response->content) unless $r->header_only;
 
-  &OK;
+  &Apache::Constants::OK;
 }
 
 *handle = \&handler; # just create alias
@@ -283,7 +338,7 @@ __END__
 
 =head1 NAME
 
-SOAP::Transport::HTTP - Server/Client side HTTP support for SOAP::Lite for Perl
+SOAP::Transport::HTTP - Server/Client side HTTP support for SOAP::Lite
 
 =head1 SYNOPSIS
 
@@ -368,85 +423,20 @@ handled.
 
 =back
 
-=head2 SERVICE DEPLOYMENT. STATIC AND DYNAMIC
+=head2 PROXY SETTINGS
 
-Let us scrutinize deployment process. Designing your SOAP server you 
-can consider two kind of deployment: B<static> and B<dynamic>.
-For both static and dynamic deployment you should specify C<MODULE>, 
-C<MODULE::method>, C<method> or C<PATH/>. Difference between static and dynamic
-deployment is that if module is not present it'll be loaded on
-demand. See L</SECURITY> section for detailed description.
+You can use any proxy setting you use with LWP::UserAgent modules:
 
-Example for B<static> deployment:
+ SOAP::Lite->proxy('http://endpoint.server', 
+                   proxy => ['http' => 'http://my.proxy.server']);
 
-  use SOAP::Transport::HTTP;
-  use My::Examples;           # module is preloaded 
+or
 
-  SOAP::Transport::HTTP::CGI
-    # deployed module should be present here or client will get 'access denied'
-    -> dispatch_to('My::Examples') 
-    -> handle;
+ $soap->transport->proxy('http' => 'http://my.proxy.server');
 
-Example for B<dynamic> deployment:
-
-  use SOAP::Transport::HTTP;
-  # name is unknown, module will be loaded on demand
-
-  SOAP::Transport::HTTP::CGI
-    # deployed module should be present here or client will get 'access denied'
-    -> dispatch_to('/Your/Path/To/Deployed/Modules', 'My::Examples') 
-    -> handle;
-
-For static deployment you should specify MODULE name directly. 
-For dynamic deployment you can specify name either directly (in that 
-case it'll be required with no restriction) or indirectly, with PATH
-(in that case ONLY path that'll be available will be PATH from 
-dispatch_to() parameters). For information how to handle this situation
-see L</SECURITY> section.
-
-=head2 SECURITY
-
-Due to security reasons if you choose dynamic deployment and specified 
-C<PATH/>, current path for perl modules (C<@INC>) will be disabled. 
-If you want to access other modules in your included package you have 
-several options:
-
-=over 4
-
-=item 1
-
-Switch to static linking:
-
-   use MODULE;
-   $server->dispatch_to('MODULE');
-
-It can be usable also when you want to import something specific
-from deployed modules: 
-
-   use MODULE qw(import_list);
-
-=item 2
-
-Change C<use> to C<require>. Path is unavailable only during 
-initialization part, and it's available again during execution. 
-So, if you do C<require> somewhere in your package it'll work.
-
-=item 3
-
-Same thing, but you can do: 
-
-   eval 'use MODULE qw(import_list)'; die if $@;
-
-=item 4
-
-Assign C<@INC> directory in your package and then make C<use>.
-Don't forget to put C<@INC> in C<BEGIN{}> block or it won't work:
-
-   BEGIN { @INC = qw(my_directory); use MODULE }
-
-Personally I don't like this method, better options are available.
-
-=back
+should specify proxy server for you. And if you use C<HTTP_proxy_user> 
+and C<HTTP_proxy_pass> for proxy authorization SOAP::Lite should know 
+how to handle it properly. 
 
 =head1 EXAMPLES
 
@@ -496,6 +486,67 @@ Apache.pm:
 
   1;
 
+=item Apache::Registry:
+
+httpd.conf:
+
+  Alias /mod_perl/ "/Apache/mod_perl/"
+  <Location /mod_perl>
+   SetHandler perl-script
+   PerlHandler Apache::Registry
+   PerlSendHeader On
+   Options +ExecCGI
+  </Location>
+
+soap.mod_cgi (put it in /Apache/mod_perl/ directory mentioned above)
+
+  use SOAP::Transport::HTTP;
+
+  SOAP::Transport::HTTP::CGI
+    -> dispatch_to('/Your/Path/To/Deployed/Modules', 'Module::Name', 'Module::method') 
+    -> handle
+  ;
+
+=back
+
+WARNING: dynamic deployment with Apache::Registry will fail, because 
+module will be loaded dynamically only for the first time. After that 
+it is already in the memory, that will bypass dynamic deployment and 
+produces error about denied access. Specify both PATH/ and MODULE name 
+in dispatch_to() and module will be loaded dynamically and then will work 
+as under static deployment. See examples/soap.mod_cgi for example.
+
+=head1 TROUBLESHOOTING
+
+=over 4
+
+=item Dynamic libraries are not found
+
+If you see in webserver's log file something like this: 
+
+Can't load '/usr/local/lib/perl5/site_perl/.../XML/Parser/Expat/Expat.so' 
+for module XML::Parser::Expat: dynamic linker: /usr/local/bin/perl:
+ libexpat.so.0 is NEEDED, but object does not exist at
+/usr/local/lib/perl5/.../DynaLoader.pm line 200.
+
+and you are using Apache web server, try to put into your httpd.conf
+
+ <IfModule mod_env.c>
+     PassEnv LD_LIBRARY_PATH
+ </IfModule>
+
+=item Apache is crashing with segfaults
+
+If using SOAP::Lite (or XML::Parser::Expat) in combination with mod_perl
+causes random segmentation faults in httpd processes try to configure
+Apache with:
+
+ RULE_EXPAT=no
+
+See http://archive.covalent.net/modperl/2000/04/0185.xml for more 
+details and lot of thanks to Robert Barta (rho@bigpond.net.au) for
+explaining this weird behavior.
+
 =back
 
 =head1 DEPENDENCIES
